/****************************************************************************
**
** Copyright (C) 2016 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the test suite of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:GPL-EXCEPT$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 as published by the Free Software
** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef MOCKMEDIASERVICE_H
#define MOCKMEDIASERVICE_H

#include "qmediaservice.h"
#include <QtCore/QMap>

Q_DECLARE_METATYPE(QMediaControl *)

class MockMediaService : public QMediaService
{
    Q_OBJECT
public:
    MockMediaService(QObject *parent, QMediaControl *control):
        QMediaService(parent),
        mockControl(control) {}

    MockMediaService(QObject *parent, QMap<QString, QMediaControl *> controls):
        QMediaService(parent),
        mockControl(0),
        mockControls(controls) {}

    QMediaControl* requestControl(const char *key)
    {
        if (mockControl)
            return mockControl;
        else
            return mockControls.value(key);
    }

    void releaseControl(QMediaControl*) {}

    QMediaControl *mockControl;
    QMap<QString, QMediaControl *>   mockControls;
};


#endif // MOCKMEDIASERVICE_H
